﻿
Module OpenCloseThereminicApps

    ' ==============================================================================================================
    '   OPEN ALL THEREMINC APPS (in the APP Folder and Subfolders)
    ' ==============================================================================================================
    Friend Sub OpenAllThereminicApps()
        ' ------------------------------------------------------------------
        FindThereminicApps()
        ' ------------------------------------------------------------------
        For Each s As String In ThereminicAppsList.Keys
            If AppIsRunning(s) Then Continue For
            Dim p As Process = New Process()
            p.StartInfo.WorkingDirectory = Application.StartupPath
            p.StartInfo.FileName = s
            p.StartInfo.UseShellExecute = False
            p.Start()
        Next
    End Sub

    ' ===========================================================
    '   Find Applications in the APP Folder and Subfolders
    ' ===========================================================
    Private ThereminicAppsList As Generic.Dictionary(Of String, Int32) = New Generic.Dictionary(Of String, Int32)

    Private Sub FindThereminicApps()
        Dim wildcards() As String = {"Theremino_*.exe"}
        Dim s As String = Application.StartupPath & "\"
        Dim fc As Collections.ObjectModel.ReadOnlyCollection(Of String)
        fc = My.Computer.FileSystem.GetFiles(s, FileIO.SearchOption.SearchAllSubDirectories, wildcards)
        For i As Int32 = 0 To fc.Count - 1
            s = fc(i)
            If Not s.Contains("vshost") And Not s.Contains("\sources\") And Not s.Contains("\obj\") Then
                If Not s.ToLower = System.Windows.Forms.Application.ExecutablePath.ToLower Then
                    ' ------------------------------------------------------------------ Add to OpenedProcesses dictionary (if the key is not present)
                    If Not ThereminicAppsList.ContainsKey(s) Then
                        ThereminicAppsList.Add(s, 0)
                    End If
                End If
            End If
        Next
    End Sub


    ' =============================================================================
    '   NORMALIZE FILENAME
    ' =============================================================================
    Friend Function NormalizeFilename(ByVal filename As String) As String
        ' ------------------------------------------------------------------ Prepare path
        If Not IO.Path.IsPathRooted(filename) Then
            filename = Application.StartupPath + "\" + filename
        End If
        ' ------------------------------------------------------------------ Correct path for /../..
        filename = IO.Path.GetFullPath(filename)
        ' ------------------------------------------------------------------ Normalize filename
        filename = filename.Replace("/", "\")
        Return (filename)
    End Function


    ' =============================================================================
    '   App Is Running  (testing the complete path of the running app)
    ' =============================================================================
    Friend Function AppIsRunning(ByVal FileName As String) As Boolean
        If FileName = "" Then Return False
        FileName = NormalizeFilename(FileName)
        If Not IO.File.Exists(FileName) Then Return False
        ' ------------------------------------------------------------------ Prepare ProcessName to compare
        Dim ProcessName As String = IO.Path.GetFileNameWithoutExtension(FileName)
        ' ------------------------------------------------------------------ Compare with all processes
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            Try
                ' ---------------------------------------------------------- test ProcessName for fast execution
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                ' ---------------------------------------------------------- test FileName
                If p(i).MainModule.FileName.ToLower = FileName.ToLower Then
                    Return True
                End If
            Catch
            End Try
        Next
        Return False
    End Function

    ' ==============================================================================================================
    '   CLOSE ALL OPENED PROCESSES
    ' ==============================================================================================================
    Friend Sub CloseAllOpenedProcesses()
        Dim ProcessName As String
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            For Each OpenedProc As System.Collections.Generic.KeyValuePair(Of String, Int32) In ThereminicAppsList
                ' ---------------------------------------------------------- test ProcessName for fast execution
                ProcessName = IO.Path.GetFileNameWithoutExtension(OpenedProc.Key)
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                ' ---------------------------------------------------------- test FileName
                If p(i).MainModule.FileName.ToLower = OpenedProc.Key.ToLower Then
                    '
                    'Debug.Print(p(i).ProcessName)
                    '
                    p(i).CloseMainWindow()
                    p(i).Close()
                    Exit For
                End If
            Next
        Next
        '
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            '
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            For Each OpenedProc As System.Collections.Generic.KeyValuePair(Of String, Int32) In ThereminicAppsList
                ' ---------------------------------------------------------- test ProcessName for fast execution
                ProcessName = IO.Path.GetFileNameWithoutExtension(OpenedProc.Key)
                If Not p(i).ProcessName.ToLower = ProcessName.ToLower Then
                    Continue For
                End If
                ' ---------------------------------------------------------- test FileName
                If Not p(i).HasExited Then
                    If p(i).MainModule.FileName.ToLower = OpenedProc.Key.ToLower Then
                        '
                        'Debug.Print(p(i).ProcessName)
                        '
                        If Not p(i).WaitForExit(1000) Then
                            Try
                                p(i).Kill()
                            Catch ex As Exception
                            End Try
                        End If
                        Exit For
                    End If
                End If
            Next
        Next
    End Sub

End Module
